function [region,curv1,curv2,map] = hermint2D(varargin)

%function hermint2D(exp,degree,n_expr,par1,par2,int1,int2)

exp = varargin{1};
degree = varargin{2};
n_expr = varargin{3};
par1 = varargin{4};
par2 = varargin{5};
X = varargin{6};
Y = varargin{7};
switch nargin
    case 8
        int = varargin{8};
    case 9
        int1 = varargin{8};
        int2 = varargin{9};
end

% Build Hermite-Fujiwara matrix H
switch degree
    case 1 
%         H = [ a1*a0 ];
        deter{1} = 'a1.*a0';
        cr1 = [exp{1}];
        cr2 = [exp{2}];
        
    case 2 
%         H = [ a1*a0 ,   0 
%                 0   , a2*a1 ];
        deter{1} = 'a1.*a0';
        deter{2} = 'a1.^2.*a0.*a2';
        cr1 = [exp{1} '.*' exp{n_expr}];
        cr2 = [exp{2}];

    case 3 
%         H = [a1*a0 ,       0      , a3*a0 
%                0   , -a3*a0+a2*a1 ,   0   
%              a3*a0 ,       0      , a3*a2]
        deter{1} = 'a1.*a0';
        deter{2} = '-(a3.*a0-a2.*a1).*a1.*a0';
        deter{3} = 'a3.*a0.*(a3.*a0-a2.*a1).^2';  
        cr1 = [exp{1} '.*' exp{n_expr}];
        cr2 = [exp{n_expr} '.*' exp{1} '-' exp{3} '.*' exp{2}];
        
    case 4 
%         H = [ a1*a0 ,       0      ,        a3*a0 ,          0
%                 0   , -a3*a0+a2*a1 ,            0 ,        a1*a4
%               a3*a0 ,       0      , -a1*a4+a3*a2 ,          0
%                 0   ,     a1*a4    ,       0      ,        a3*a4]
        deter{1} = 'a1.*a0';
        deter{2} = '-(a3.*a0-a2.*a1).*a1.*a0';
        deter{3} = 'a0.*(a3.*a0-a2.*a1).*(a4.*a1.^2-a3.*a2.*a1+a3.^2.*a0)';                    
        deter{4} = 'a0.*a4.*(a4.*a1.^2-a3.*a2.*a1+a3.^2.*a0).^2';
        cr1 = [exp{1} '.*' exp{n_expr}];
        cr2 = [exp{n_expr} '.*' exp{2} '.^2-' exp{4} '.*' exp{3} '.*' exp{2} '+' exp{4} '.^2.*' exp{1}];
    case 5 
%         H = [a0*a1 ,       0,             a0*a3       ,       0      , a0*a5
%                0   , -a0*a3+a1*a2 ,         0         , -a0*a5+a1*a4 ,   0
%              a0*a3 ,       0      , a0*a5-a1*a4+a2*a3 ,       0      , a2*a5
%                0   , -a0*a5+a1*a4 ,         0         , -a2*a5+a3*a4 ,   0
%              a0*a5 ,       0,             a2*a5       ,       0      , a4*a5]
        deter{1} = 'a1.*a0';
        deter{2} = '-(a3.*a0-a2.*a1).*a1.*a0';
        deter{3} = 'a0.*(a3.*a0-a2.*a1).*(a3.^2.*a0-a0.*a1.*a5+a4.*a1.^2-a3.*a2.*a1)';                    
        deter{4} = 'a0.*(a3.^2.*a0-a0.*a1.*a5+a4.*a1.^2-a3.*a2.*a1).*(a0.^2.*a5.^2-a3.*a0.*a2.*a5+a3.^2.*a0.*a4-2.*a0.*a5.*a1.*a4+a2.^2.*a1.*a5-a2.*a1.*a3.*a4+a1.^2.*a4.^2)';  
        deter{5} = 'a0.*a5.*(a0.^2.*a5.^2-a3.*a0.*a2.*a5+a3.^2.*a0.*a4-2.*a0.*a5.*a1.*a4+a2.^2.*a1.*a5-a2.*a1.*a3.*a4+a1.^2.*a4.^2).^2';        
        cr1 = [exp{1} '.*' exp{n_expr}];
        cr2 = [exp{1} '.^2.*' exp{n_expr} '.^2 - ' exp{4} '.*' exp{1} '.*' exp{3} '.*' exp{n_expr} '+' exp{4} '.^2.*' exp{1} '.*' exp{5} ...
                '-2.*' exp{1} '.*' exp{n_expr} '.*' exp{2} '.*' exp{5} '+' exp{3} '.^2.*' exp{2} '.*' exp{n_expr} '-' exp{3} '.*' exp{2} ...
                '.*' exp{4} '.*' exp{5} '+' exp{2} '.^2.*' exp{5} '.^2'];
    case 6
% [             a0*a1,                 0,             a0*a3,                 0,             a5*a0,                 0]
% [                 0,      -a0*a3+a1*a2,                 0,      -a5*a0+a4*a1,                 0,             a6*a1]
% [             a0*a3,                 0, a5*a0-a4*a1+a2*a3,                 0,      -a6*a1+a2*a5,                 0]
% [                 0,      -a5*a0+a4*a1,                 0, a6*a1-a2*a5+a3*a4,                 0,             a3*a6]
% [             a5*a0,                 0,      -a6*a1+a2*a5,                 0,      -a3*a6+a4*a5,                 0]
% [                 0,             a6*a1,                 0,             a3*a6,                 0,             a6*a5]        
        deter{1} = 'a1.*a0';
        deter{2} = '-a0.*a1.*(a0.*a3-a1.*a2)';
        deter{3} = 'a0.*(a0.*a3-a1.*a2).*(a0.*a3.^2-a1.*a2.*a3-a1.*a5.*a0+a1.^2.*a4)';
        deter{4} = 'a0.*(a0.*a3.^2-a1.*a2.*a3-a1.*a5.*a0+a1.^2.*a4).*(a0.*a3.^2.*a4+a0.*a3.*a6.*a1-a0.*a3.*a2.*a5-a1.*a2.*a3.*a4+a5.^2.*a0.^2-2.*a5.*a0.*a4.*a1-a1.^2.*a2.*a6+a4.^2.*a1.^2+a1.*a2.^2.*a5)';
        deter{5} = '-a0.*(a0.*a3.^2.*a4+a0.*a3.*a6.*a1-a0.*a3.*a2.*a5-a1.*a2.*a3.*a4+a5.^2.*a0.^2-2.*a5.*a0.*a4.*a1-a1.^2.*a2.*a6+a4.^2.*a1.^2+a1.*a2.^2.*a5).*(-a5.^3.*a0.^2+2.*a0.*a4.*a1.*a5.^2+a0.*a2.*a5.^2.*a3-a0.*a4.*a5.*a3.^2-3.*a6.*a1.*a0.*a5.*a3+a6.*a0.*a3.^3-a2.^2.*a1.*a5.^2-a4.^2.*a1.^2.*a5+a2.*a4.*a1.*a5.*a3+2.*a1.^2.*a2.*a6.*a5+a6.*a4.*a1.^2.*a3-a1.^3.*a6.^2-a1.*a2.*a6.*a3.^2)';
        deter{6} = 'a6.*a0.*(-a5.^3.*a0.^2+2.*a0.*a4.*a1.*a5.^2+a0.*a2.*a5.^2.*a3-a0.*a4.*a5.*a3.^2-3.*a6.*a1.*a0.*a5.*a3+a6.*a0.*a3.^3-a2.^2.*a1.*a5.^2-a4.^2.*a1.^2.*a5+a2.*a4.*a1.*a5.*a3+2.*a1.^2.*a2.*a6.*a5+a6.*a4.*a1.^2.*a3-a1.^3.*a6.^2-a1.*a2.*a6.*a3.^2).^2';
        cr1 = [exp{1} '.*' exp{n_expr}];
        cr2 = ['-' exp{6} '.^3.*' exp{1} '.^2+2.*' exp{1} '.*' exp{5} '.*' exp{2} '.*' exp{6} '.^2+' exp{1} '.*' exp{3} '.*' exp{6} '.^2.*' ...
                exp{4} '-' exp{1} '.*' exp{5} '.*' exp{6} '.*' exp{4} '.^2-3.*' exp{7} '.*' exp{2} '.*' exp{1} '.*' exp{6} '.*' exp{4} '+' ...
                exp{7} '.*' exp{1} '.*' exp{4} '.^3-' exp{3} '.^2.*' exp{2} '.*' exp{6} '.^2-' exp{5} '.^2.*' exp{2} '.^2.*' exp{4} '+' ...
                exp{3} '.*' exp{5} '.*' exp{2} '.*' exp{6} '.*' exp{4} '+2.*' exp{2} '.^2.*' exp{3} '.*' exp{7} '.*' exp{6} '+' exp{7} '.*' ...
                exp{5} '.*' exp{2} '.^2.*' exp{4} '-' exp{2} '.^3.*' exp{7} '.^2-' exp{2} '.*' exp{3} '.*' exp{7} '.*' exp{4} '.^2'];
    case 7
% [                    a0*a1,                        0,                    a0*a3,                        0,                    a5*a0,                        0,                    a7*a0]
% [                        0,             -a0*a3+a1*a2,                        0,             -a5*a0+a4*a1,                        0,             -a7*a0+a6*a1,                        0]
% [                    a0*a3,                        0,        a5*a0-a4*a1+a2*a3,                        0,        a7*a0-a6*a1+a2*a5,                        0,                    a2*a7]
% [                        0,             -a5*a0+a4*a1,                        0, -a7*a0+a6*a1-a2*a5+a3*a4,                        0,             -a2*a7+a3*a6,                        0]
% [                    a5*a0,                        0,        a7*a0-a6*a1+a2*a5,                        0,        a2*a7-a3*a6+a4*a5,                        0,                    a4*a7]
% [                        0,             -a7*a0+a6*a1,                        0,             -a2*a7+a3*a6,                        0,             -a4*a7+a5*a6,                        0]
% [                    a7*a0,                        0,                    a2*a7,                        0,                    a4*a7,                        0,                    a6*a7]
        deter{1} = 'a1.*a0';
        deter{2} = '-a0.*a1.*(a0.*a3-a1.*a2)';
        deter{3} = 'a0.*(a0.*a3-a1.*a2).*(a0.*a3.^2-a1.*a2.*a3-a1.*a5.*a0+a1.^2.*a4)';
        deter{4} = 'a0.*(a0.*a3.^2-a1.*a2.*a3-a1.*a5.*a0+a1.^2.*a4).*(a0.*a3.^2.*a4-a0.^2.*a3.*a7-a0.*a3.*a2.*a5+a0.*a3.*a6.*a1-a1.*a2.*a3.*a4+a5.^2.*a0.^2+a1.*a2.*a7.*a0-2.*a5.*a0.*a4.*a1+a1.*a2.^2.*a5-a1.^2.*a2.*a6+a4.^2.*a1.^2)';
        deter{5} = '-a0.*(a0.*a3.^2.*a4-a0.^2.*a3.*a7-a0.*a3.*a2.*a5+a0.*a3.*a6.*a1-a1.*a2.*a3.*a4+a5.^2.*a0.^2+a1.*a2.*a7.*a0-2.*a5.*a0.*a4.*a1+a1.*a2.^2.*a5-a1.^2.*a2.*a6+a4.^2.*a1.^2).*(a6.*a0.*a3.^3-a4.*a5.*a0.*a3.^2-a2.*a7.*a0.*a3.^2+a5.^2.*a2.*a0.*a3-3.*a5.*a6.*a1.*a0.*a3+2.*a5.^2.*a4.*a1.*a0-a5.*a1.*a2.*a7.*a0+2.*a1.^2.*a7.*a6.*a0+a4.*a1.^2.*a6.*a3-a1.^2.*a2.*a7.*a4+2.*a5.*a1.^2.*a2.*a6+a5.*a4.*a1.*a2.*a3-a1.^3.*a6.^2-a1.*a2.*a6.*a3.^2+a1.*a2.^2.*a7.*a3-a1.*a7.^2.*a0.^2+2.*a5.*a7.*a0.^2.*a3-a5.^3.*a0.^2-a5.^2.*a1.*a2.^2-a5.*a4.^2.*a1.^2)';
        deter{6} = 'a0.*(a6.*a0.*a3.^3-a4.*a5.*a0.*a3.^2-a2.*a7.*a0.*a3.^2+a5.^2.*a2.*a0.*a3-3.*a5.*a6.*a1.*a0.*a3+2.*a5.^2.*a4.*a1.*a0-a5.*a1.*a2.*a7.*a0+2.*a1.^2.*a7.*a6.*a0+a4.*a1.^2.*a6.*a3-a1.^2.*a2.*a7.*a4+2.*a5.*a1.^2.*a2.*a6+a5.*a4.*a1.*a2.*a3-a1.^3.*a6.^2-a1.*a2.*a6.*a3.^2+a1.*a2.^2.*a7.*a3-a1.*a7.^2.*a0.^2+2.*a5.*a7.*a0.^2.*a3-a5.^3.*a0.^2-a5.^2.*a1.*a2.^2-a5.*a4.^2.*a1.^2).*(-3.*a1.*a7.^2.*a6.*a0.^2-a5.^3.*a6.*a0.^2-a2.*a7.^2.*a5.*a0.^2-a1.*a5.*a2.*a6.*a7.*a0+3.*a1.^2.*a7.*a6.^2.*a0+a7.^3.*a0.^3-2.*a1.*a5.*a7.*a0.*a4.^2-a1.^2.*a5.*a6.*a4.^2+a1.^2.*a4.^3.*a7-a1.*a2.^3.*a7.^2-3.*a1.^2.*a2.*a7.*a6.*a4+a7.*a5.^2.*a0.^2.*a4+3.*a1.*a2.*a7.^2.*a0.*a4+a1.*a2.^2.*a5.*a7.*a4+3.*a7.*a5.*a6.*a0.^2.*a3+a2.*a5.^2.*a6.*a0.*a3+a2.^2.*a7.^2.*a0.*a3-a1.*a2.*a7.*a4.^2.*a3-3.*a1.*a5.*a6.^2.*a0.*a3+a7.*a0.*a4.^2.*a3.^2+a1.^2.*a6.^2.*a4.*a3-2.*a7.^2.*a0.^2.*a4.*a3+a1.*a5.*a2.*a6.*a4.*a3-a2.*a5.*a7.*a0.*a4.*a3+a1.*a7.*a6.*a0.*a4.*a3+2.*a1.*a2.^2.*a7.*a6.*a3-a1.*a2.*a6.^2.*a3.^2-a6.*a5.*a0.*a4.*a3.^2-2.*a2.*a7.*a6.*a0.*a3.^2+a6.^2.*a0.*a3.^3+2.*a1.*a5.^2.*a6.*a0.*a4-a1.*a2.^2.*a5.^2.*a6+2.*a1.^2.*a5.*a6.^2.*a2-a1.^3.*a6.^3)';
        deter{7} = 'a7.*a0.*(-3.*a1.*a7.^2.*a6.*a0.^2-a5.^3.*a6.*a0.^2-a2.*a7.^2.*a5.*a0.^2-a1.*a5.*a2.*a6.*a7.*a0+3.*a1.^2.*a7.*a6.^2.*a0+a7.^3.*a0.^3-2.*a1.*a5.*a7.*a0.*a4.^2-a1.^2.*a5.*a6.*a4.^2+a1.^2.*a4.^3.*a7-a1.*a2.^3.*a7.^2-3.*a1.^2.*a2.*a7.*a6.*a4+a7.*a5.^2.*a0.^2.*a4+3.*a1.*a2.*a7.^2.*a0.*a4+a1.*a2.^2.*a5.*a7.*a4+3.*a7.*a5.*a6.*a0.^2.*a3+a2.*a5.^2.*a6.*a0.*a3+a2.^2.*a7.^2.*a0.*a3-a1.*a2.*a7.*a4.^2.*a3-3.*a1.*a5.*a6.^2.*a0.*a3+a7.*a0.*a4.^2.*a3.^2+a1.^2.*a6.^2.*a4.*a3-2.*a7.^2.*a0.^2.*a4.*a3+a1.*a5.*a2.*a6.*a4.*a3-a2.*a5.*a7.*a0.*a4.*a3+a1.*a7.*a6.*a0.*a4.*a3+2.*a1.*a2.^2.*a7.*a6.*a3-a1.*a2.*a6.^2.*a3.^2-a6.*a5.*a0.*a4.*a3.^2-2.*a2.*a7.*a6.*a0.*a3.^2+a6.^2.*a0.*a3.^3+2.*a1.*a5.^2.*a6.*a0.*a4-a1.*a2.^2.*a5.^2.*a6+2.*a1.^2.*a5.*a6.^2.*a2-a1.^3.*a6.^3).^2';
        cr1 = [exp{1} '.*' exp{n_expr}];
        cr2 = ['-3.*' exp{2} '.*' exp{8} '.^2.*' exp{7} '.*' exp{1} '.^2-' exp{6} '.^3.*' exp{7} '.*' exp{1} '.^2-' exp{3} '.*' exp{8} '.^2.*' ... 
                exp{6} '.*' exp{1} '.^2-' exp{2} '.*' exp{4} '.*' exp{3} '.*' exp{7} '.*' exp{8} '.*' exp{1} '+3.*' exp{2} '.^2.*' exp{8} '.*' ...
                exp{7} '.^2.*' exp{1} '+' exp{8} '.^3.*' exp{1} '.^3-2.*' exp{2} '.*' exp{6} '.*' exp{8} '.*' exp{1} '.*' exp{5} '.^2-' exp{2} ...
                '.^2.*' exp{6} '.*' exp{7} '.*' exp{5} '.^2+' exp{2} '.^2.*' exp{5} '.^3.*' exp{8} '-' exp{2} '.*' exp{3} '.^3.*' exp{8} '.^2-3.*' ...
                exp{2} '.^2.*' exp{3} '.*' exp{8} '.*' exp{7} '.*' exp{5} '+' exp{8} '.*' exp{6} '.^2.*' exp{1} '.^2.*' exp{5} '+3.*' exp{2} '.*' ...
                exp{3} '.*' exp{8} '.^2.*' exp{1} '.*' exp{5} '+' exp{2} '.*' exp{3} '.^2.*' exp{6} '.*' exp{8} '.*' exp{5} '+3.*' exp{8} '.*' ...
                exp{6} '.*' exp{7} '.*' exp{1} '.^2.*' exp{4} '+' exp{3} '.*' exp{6} '.^2.*' exp{7} '.*' exp{1} '.*' exp{4} '+' exp{3} '.^2.*' ...
                exp{8} '.^2.*' exp{1} '.*' exp{4} '-' exp{2} '.*' exp{3} '.*' exp{8} '.*' exp{5} '.^2.*' exp{4} '-3.*' exp{2} '.*' exp{6} '.*' ...
                exp{7} '.^2.*' exp{1} '.*' exp{4} '+' exp{8} '.*' exp{1} '.*' exp{5} '.^2.*' exp{4} '.^2+' exp{2} '.^2.*' exp{7} '.^2.*' exp{5} ...
                '.*' exp{4} '-2.*' exp{8} '.^2.*' exp{1} '.^2.*' exp{5} '.*' exp{4} '+' exp{2} '.*' exp{6} '.*' exp{3} '.*' exp{7} '.*' exp{5} ...
                '.*' exp{4} '-' exp{3} '.*' exp{6} '.*' exp{8} '.*' exp{1} '.*' exp{5} '.*' exp{4} '+' exp{2} '.*' exp{8} '.*' exp{7} '.*' exp{1} ...
                '.*' exp{5} '.*' exp{4} '+2.*' exp{2} '.*' exp{3} '.^2.*' exp{8} '.*' exp{7} '.*' exp{4} '-' exp{2} '.*' exp{3} '.*' exp{7} '.^2.*' ...
                exp{4} '.^2-' exp{7} '.*' exp{6} '.*' exp{1} '.*' exp{5} '.*' exp{4} '.^2-2.*' exp{3} '.*' exp{8} '.*' exp{7} '.*' exp{1} '.*' exp{4} ...
                '.^2+' exp{7} '.^2.*' exp{1} '.*' exp{4} '.^3+2.*' exp{2} '.*' exp{6} '.^2.*' exp{7} '.*' exp{1} '.*' exp{5} '-' exp{2} '.*' exp{3} ...
                '.^2.*' exp{6} '.^2.*' exp{7} '+2.*' exp{2} '.^2.*' exp{6} '.*' exp{7} '.^2.*' exp{3} '-' exp{2} '.^3.*' exp{7} '.^3' ];
end

for ii = 1 : degree
    for jj = 0 : n_expr - 1
        deter{ii} = strrep(deter{ii},sprintf('a%d',jj),exp{jj+1});
    end
end

for ii = 1 : degree
    cur{ii} = eval(deter{ii});
    cur{ii} = sign(cur{ii})>0;
    if ii == 1
        oblast = cur{ii};
    else
        oblast = oblast.*cur{ii};
    end
end

curv1 = eval(cr1);
curv2 = eval(cr2);

%figure;
%pcolor(X,Y,oblast);
region = pcolor(X,Y,double(oblast));
%region=6;
map = [1.0000    1.0000    1.0000
       1.0000    1.0000    1.0000
       1.0000    1.0000    1.0000
       1.0000    1.0000    1.0000
       1.0000    1.0000    1.0000
       1.0000    1.0000    1.0000
       1.0000    1.0000    1.0000
       1.0000    1.0000    1.0000
       1.0000    1.0000    1.0000
       0.9289    1.0000    1.0000
       0.8577    1.0000    1.0000
       0.7866    1.0000    1.0000
       0.7154    1.0000    1.0000
       0.6443    1.0000    1.0000
       0.5731    1.0000    1.0000
       0.5020    1.0000    1.0000];
colormap(map); %prism
hold on;
shading flat; %faceted; %interp; %flat;
if (size(curv1,1) ~= 1) & (size(curv1,2) ~= 1)
    [c,curv1] = contour(X,Y,curv1,[0 0],'-b');
else
    curv1 = plot(int1,curv1,'-b');
end
hold on;
if (size(curv2,1) ~= 1) & (size(curv2,2) ~= 1)
    [c,curv2] = contour(X,Y,curv2,[0 0],'-b');
else
    curv2 = plot(int1,curv2,'-b');
end

%title('Stability area','FontSize',[12],'FontWeight','bold');
%set(gca,'Color','none');
xlabel(char(par1),'FontSize',[12],'FontWeight','bold');
ylabel(char(par2),'FontSize',[12],'FontWeight','bold');
hold off;